@@ -1,13 +1,12 @@
-// Copyright 2010 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
-//
+/**
+ * @license
+ * Copyright 2010 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
+
 // Various tools for parsing LLVM. Utilities of various sorts, that are
 // specific to Emscripten (and hence not in utility.js).
 
-//"use strict";
-
 // Does simple 'macro' substitution, using Django-like syntax,
 // {{{ code }}} will be replaced with |eval(code)|.
 // NOTE: Be careful with that ret check. If ret is |0|, |ret ? ret.toString() : ''| would result in ''!
@@ -1529,18 +1528,18 @@ function getQuoted(str) {
 }
 
 function makeRetainedCompilerSettings() {
-  var blacklist = set('STRUCT_INFO');
+  var ignore = set('STRUCT_INFO');
   if (STRICT) {
     for (var i in LEGACY_SETTINGS) {
       var name = LEGACY_SETTINGS[i][0];
-      blacklist[name] = 0;
+      ignore[name] = 0;
     }
   }
 
   var ret = {};
   for (var x in this) {
     try {
-      if (x[0] !== '_' && !(x in blacklist) && x == x.toUpperCase() && (typeof this[x] === 'number' || typeof this[x] === 'string' || this.isArray())) ret[x] = this[x];
+      if (x[0] !== '_' && !(x in ignore) && x == x.toUpperCase() && (typeof this[x] === 'number' || typeof this[x] === 'string' || this.isArray())) ret[x] = this[x];
     } catch(e){}
   }
   return ret;
@@ -1604,7 +1603,7 @@ function expectToReceiveOnModule(name) {
 function makeRemovedModuleAPIAssert(moduleName, localName) {
   if (!ASSERTIONS) return '';
   if (!localName) localName = moduleName;
-  return "if (!Object.getOwnPropertyDescriptor(Module, '" + moduleName + "')) Object.defineProperty(Module, '" + moduleName + "', { configurable: true, get: function() { abort('Module." + moduleName + " has been replaced with plain " + localName + "') } });";
+  return "if (!Object.getOwnPropertyDescriptor(Module, '" + moduleName + "')) Object.defineProperty(Module, '" + moduleName + "', { configurable: true, get: function() { abort('Module." + moduleName + " has been replaced with plain " + localName + " (the initial value can be provided on Module, but after startup the value is only looked for on a local variable of that name)') } });";
 }
 
 // Make code to receive a value on the incoming Module object.
@@ -1678,3 +1677,57 @@ function makeAsmImportsAccessInPthread(variable) {
 function hasExportedFunction(func) {
   return Object.keys(EXPORTED_FUNCTIONS).indexOf(func) != -1;
 }
+
+// JS API I64 param handling: if we have BigInt support, the ABI is simple,
+// it is a BigInt. Otherwise, we legalize into pairs of i32s.
+function defineI64Param(name) {
+  if (WASM_BIGINT) {
+    return name + '_bigint';
+  } else {
+    return name + '_low, ' + name + '_high';
+  }
+}
+
+function receiveI64ParamAsI32s(name) {
+  if (WASM_BIGINT) {
+    // TODO: use Xn notation when JS parsers support it (as of April 6 2020,
+    //  * closure compiler is missing support
+    //    https://github.com/google/closure-compiler/issues/3167
+    //  * acorn needs to be upgraded, and to set ecmascript version >= 11
+    //  * terser needs to be upgraded
+    return 'var ' + name + '_low = Number(' + name + '_bigint & BigInt(0xffffffff)) | 0, ' + name + '_high = Number(' + name + '_bigint >> BigInt(32)) | 0;';
+  } else {
+    return '';
+  }
+}
+
+function sendI64Argument(low, high) {
+  if (WASM_BIGINT) {
+    return 'BigInt(low) | (BigInt(high) << BigInt(32))';
+  } else {
+    return low + ', ' + high;
+  }
+}
+
+// Add assertions to catch common errors when using the Promise object we
+// create on Module.ready() and return from MODULARIZE Module() invocations.
+function addReadyPromiseAssertions(promise) {
+  // Warn on someone doing
+  //
+  //  var instance = Module();
+  //  ...
+  //  instance._main();
+  var properties = keys(EXPORTED_FUNCTIONS);
+  // Also warn on onRuntimeInitialized which might be a common pattern with
+  // older MODULARIZE-using codebases.
+  properties.push('onRuntimeInitialized');
+  return properties.map(function(property) {
+    const warningEnding = `${property} on the Promise object, instead of the instance. Use .then() to get called back with the instance, see the MODULARIZE docs in src/settings.js`;
+    return `
+      if (!Object.getOwnPropertyDescriptor(${promise}, '${property}')) {
+        Object.defineProperty(${promise}, '${property}', { configurable: true, get: function() { abort('You are getting ${warningEnding}') } });
+        Object.defineProperty(${promise}, '${property}', { configurable: true, set: function() { abort('You are setting ${warningEnding}') } });
+      }
+    `;
+  }).join('\n');
+}
