@@ -39,24 +39,26 @@ i32x4 TESTFN v32x4_load_splat(int32_t *ptr) {
 i64x2 TESTFN v64x2_load_splat(int64_t *ptr) {
   return (i64x2){*ptr, *ptr};
 }
-i16x8 TESTFN i16x8_load8x8_s(i8x8 *ptr) {
-  return __builtin_convertvector(*ptr, i16x8);
-}
-i16x8 TESTFN i16x8_load8x8_u(i8x8 *ptr) {
-  return (i16x8)__builtin_convertvector(*(u8x8*)ptr, u16x8);
-}
-i32x4 TESTFN i32x4_load16x4_s(i16x4 *ptr) {
-  return __builtin_convertvector(*ptr, i32x4);
-}
-i32x4 TESTFN i32x4_load16x4_u(i16x4 *ptr) {
-  return (i32x4)__builtin_convertvector(*(u16x4*)ptr, u32x4);
-}
-i64x2 TESTFN i64x2_load32x2_s(i32x2 *ptr) {
-  return __builtin_convertvector(*ptr, i64x2);
-}
-i64x2 TESTFN i64x2_load32x2_u(i32x2 *ptr) {
-  return (i64x2) __builtin_convertvector(*(u32x2*)ptr, u64x2);
-}
+// The tools and engine disagree on what the allowed alignments should be, so
+// skip these tests for now: https://github.com/WebAssembly/simd/issues/230
+// i16x8 TESTFN i16x8_load8x8_s(i8x8 *ptr) {
+//   return __builtin_convertvector(*ptr, i16x8);
+// }
+// i16x8 TESTFN i16x8_load8x8_u(i8x8 *ptr) {
+//   return (i16x8)__builtin_convertvector(*(u8x8*)ptr, u16x8);
+// }
+// i32x4 TESTFN i32x4_load16x4_s(i16x4 *ptr) {
+//   return __builtin_convertvector(*ptr, i32x4);
+// }
+// i32x4 TESTFN i32x4_load16x4_u(i16x4 *ptr) {
+//   return (i32x4)__builtin_convertvector(*(u16x4*)ptr, u32x4);
+// }
+// i64x2 TESTFN i64x2_load32x2_s(i32x2 *ptr) {
+//   return __builtin_convertvector(*ptr, i64x2);
+// }
+// i64x2 TESTFN i64x2_load32x2_u(i32x2 *ptr) {
+//   return (i64x2) __builtin_convertvector(*(u32x2*)ptr, u64x2);
+// }
 void TESTFN i8x16_store(i8x16 *ptr, i8x16 vec) {
   *ptr = vec;
 }
@@ -69,11 +71,9 @@ i8x16 TESTFN i8x16_shuffle_interleave_bytes(i8x16 x, i8x16 y) {
 i32x4 TESTFN i32x4_shuffle_reverse(i32x4 vec) {
   return __builtin_shufflevector(vec, vec, 3, 2, 1, 0);
 }
-#ifdef __wasm_unimplemented_simd128__
 i8x16 TESTFN v8x16_swizzle(i8x16 x, i8x16 y) {
   return __builtin_wasm_swizzle_v8x16(x, y);
 }
-#endif // __wasm_unimplemented_simd128__
 i8x16 TESTFN i8x16_splat(int32_t x) {
   return (i8x16) {x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x};
 }
@@ -320,6 +320,9 @@ i8x16 TESTFN i8x16_andnot(i8x16 x, i8x16 y) {
 i8x16 TESTFN i8x16_bitselect(i8x16 x, i8x16 y, i8x16 cond) {
   return (i8x16)__builtin_wasm_bitselect((i32x4)x, (i32x4)y, (i32x4)cond);
 }
+i8x16 TESTFN i8x16_abs(i8x16 vec) {
+  return (i8x16)__builtin_wasm_abs_i8x16(vec);
+}
 i8x16 TESTFN i8x16_neg(i8x16 vec) {
   return -vec;
 }
@@ -356,13 +359,13 @@ i8x16 TESTFN i8x16_sub_saturate_s(i8x16 x, i8x16 y) {
 i8x16 TESTFN i8x16_sub_saturate_u(i8x16 x, i8x16 y) {
   return __builtin_wasm_sub_saturate_u_i8x16(x, y);
 }
-i8x16 TESTFN i8x16_mul(i8x16 x, i8x16 y) {
-  return x * y;
-}
-// Skip {min,max}_{s,u} because they do not have short builtin equivalents
+// TODO: min_s / min_u / max_s / max_u
 i8x16 TESTFN i8x16_avgr_u(i8x16 x, i8x16 y) {
   return __builtin_wasm_avgr_u_i8x16(x, y);
 }
+i16x8 TESTFN i16x8_abs(i16x8 vec) {
+  return (i16x8)__builtin_wasm_abs_i16x8(vec);
+}
 i16x8 TESTFN i16x8_neg(i16x8 vec) {
   return -vec;
 }
@@ -402,10 +405,13 @@ i16x8 TESTFN i16x8_sub_saturate_u(i16x8 x, i16x8 y) {
 i16x8 TESTFN i16x8_mul(i16x8 x, i16x8 y) {
   return x * y;
 }
-// Skip {min,max}_{s,u} because they do not have short builtin equivalents
+// TODO: min_s / min_u / max_s / max_u
 i16x8 TESTFN i16x8_avgr_u(i16x8 x, i16x8 y) {
   return __builtin_wasm_avgr_u_i16x8(x, y);
 }
+i32x4 TESTFN i32x4_abs(i32x4 vec) {
+  return (i32x4)__builtin_wasm_abs_i32x4(vec);
+}
 i32x4 TESTFN i32x4_neg(i32x4 vec) {
   return -vec;
 }
@@ -433,7 +439,7 @@ i32x4 TESTFN i32x4_sub(i32x4 x, i32x4 y) {
 i32x4 TESTFN i32x4_mul(i32x4 x, i32x4 y) {
   return x * y;
 }
-// Skip {min,max}_{s,u} because they do not have short builtin equivalents
+// TODO: min_s / min_u / max_s / max_u
 i64x2 TESTFN i64x2_neg(i64x2 vec) {
   return -vec;
 }
@@ -536,26 +542,12 @@ i32x4 TESTFN i32x4_trunc_s_f32x4_sat(f32x4 vec) {
 i32x4 TESTFN i32x4_trunc_u_f32x4_sat(f32x4 vec) {
   return __builtin_wasm_trunc_saturate_u_i32x4_f32x4(vec);
 }
-#ifdef __wasm_unimplemented_simd128__
-i64x2 TESTFN i64x2_trunc_s_f64x2_sat(f64x2 vec) {
-  return __builtin_wasm_trunc_saturate_s_i64x2_f64x2(vec);
-}
-i64x2 TESTFN i64x2_trunc_u_f64x2_sat(f64x2 vec) {
-  return __builtin_wasm_trunc_saturate_u_i64x2_f64x2(vec);
-}
-#endif // __wasm_unimplemented_simd128__
 f32x4 TESTFN f32x4_convert_s_i32x4(i32x4 vec) {
   return __builtin_convertvector(vec, f32x4);
 }
 f32x4 TESTFN f32x4_convert_u_i32x4(i32x4 vec) {
   return __builtin_convertvector((u32x4)vec, f32x4);
 }
-f64x2 TESTFN f64x2_convert_s_i64x2(i64x2 vec) {
-  return __builtin_convertvector(vec, f64x2);
-}
-f64x2 TESTFN f64x2_convert_u_i64x2(i64x2 vec) {
-  return __builtin_convertvector((u64x2)vec, f64x2);
-}
 i8x16 TESTFN i8x16_narrow_i16x8_s(i16x8 a, i16x8 b) {
   return __builtin_wasm_narrow_s_i8x16_i16x8(a, b);
 }
@@ -689,30 +681,30 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     );
     expect_vec(v32x4_load_splat((int32_t*)&vec), ((i32x4){0xb0a09080, 0xb0a09080, 0xb0a09080, 0xb0a09080}));
     expect_vec(v64x2_load_splat((int64_t*)&vec), ((i64x2){0xf0e0d0c0b0a09080, 0xf0e0d0c0b0a09080}));
-    expect_vec(
-      i16x8_load8x8_s(&vec),
-      ((i16x8){0xff80, 0xff90, 0xffa0, 0xffb0, 0xffc0, 0xffd0, 0xffe0, 0xfff0})
-    );
-    expect_vec(
-      i16x8_load8x8_u(&vec),
-      ((i16x8){0x0080, 0x0090, 0x00a0, 0x00b0, 0x00c0, 0x00d0, 0x00e0, 0x00f0})
-    );
-    expect_vec(
-      i32x4_load16x4_s((i16x4*)&vec),
-      ((i32x4){0xffff9080, 0xffffb0a0, 0xffffd0c0, 0xfffff0e0})
-    );
-    expect_vec(
-      i32x4_load16x4_u((i16x4*)&vec),
-      ((i32x4){0x00009080, 0x0000b0a0, 0x0000d0c0, 0x0000f0e0})
-    );
-    expect_vec(
-      i64x2_load32x2_s((i32x2*)&vec),
-      ((i64x2){0xffffffffb0a09080, 0xfffffffff0e0d0c0})
-    );
-    expect_vec(
-      i64x2_load32x2_u((i32x2*)&vec),
-      ((i64x2){0x00000000b0a09080, 0x00000000f0e0d0c0})
-    );
+    // expect_vec(
+    //   i16x8_load8x8_s(&vec),
+    //   ((i16x8){0xff80, 0xff90, 0xffa0, 0xffb0, 0xffc0, 0xffd0, 0xffe0, 0xfff0})
+    // );
+    // expect_vec(
+    //   i16x8_load8x8_u(&vec),
+    //   ((i16x8){0x0080, 0x0090, 0x00a0, 0x00b0, 0x00c0, 0x00d0, 0x00e0, 0x00f0})
+    // );
+    // expect_vec(
+    //   i32x4_load16x4_s((i16x4*)&vec),
+    //   ((i32x4){0xffff9080, 0xffffb0a0, 0xffffd0c0, 0xfffff0e0})
+    // );
+    // expect_vec(
+    //   i32x4_load16x4_u((i16x4*)&vec),
+    //   ((i32x4){0x00009080, 0x0000b0a0, 0x0000d0c0, 0x0000f0e0})
+    // );
+    // expect_vec(
+    //   i64x2_load32x2_s((i32x2*)&vec),
+    //   ((i64x2){0xffffffffb0a09080, 0xfffffffff0e0d0c0})
+    // );
+    // expect_vec(
+    //   i64x2_load32x2_u((i32x2*)&vec),
+    //   ((i64x2){0x00000000b0a09080, 0x00000000f0e0d0c0})
+    // );
   }
   expect_vec(i32x4_const(), ((i32x4){1, 2, 3, 4}));
   expect_vec(
@@ -723,7 +715,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     ((i8x16){1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16})
   );
   expect_vec(i32x4_shuffle_reverse((i32x4){1, 2, 3, 4}), ((i32x4){4, 3, 2, 1}));
-#ifdef  __wasm_unimplemented_simd128__
   expect_vec(
     v8x16_swizzle(
       (i8x16){0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
@@ -733,7 +724,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     ((i8x16){0xf0, 0xf4, 0xf8, 0xfc, 0x00, 0x00, 0x00, 0x00,
              0x00, 0x00, 0xff, 0xfd, 0xfc, 0xf8, 0xf4, 0xf0})
   );
-#endif // __wasm_unimplemented_simd128__
 
   // i8x16 lane accesses
   expect_vec(i8x16_splat(5), ((i8x16){5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5}));
@@ -1075,6 +1065,10 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   );
 
   // i8x16 arithmetic
+  expect_vec(
+    i8x16_abs((i8x16){0, 1, 42, -3, -56, 127, -128, -126, 0, -1, -42, 3, 56, -127, -128, 126}),
+    ((i8x16){0, 1, 42, 3, 56, 127, -128, 126, 0, 1, 42, 3, 56, 127, -128, 126})
+  );
   expect_vec(
     i8x16_neg((i8x16){0, 1, 42, -3, -56, 127, -128, -126, 0, -1, -42, 3, 56, -127, -128, 126}),
     ((i8x16){0, -1, -42, 3, 56, -127, -128, 126, 0, 1, 42, -3, -56, 127, -128, -126})
@@ -1153,13 +1147,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     ),
     ((i8x16){0, 0, 254, 0, 0, 123, 0, 12, 61, 167, 158, 100, 17, 0, 0, 0})
   );
-  expect_vec(
-    i8x16_mul(
-      (i8x16){0, 42, 255, 128, 127, 129, 6, 29, 103, 196, 231, 142, 17, 250, 1, 73},
-      (i8x16){3, 231, 1, 128, 129, 6, 103, 17, 42, 29, 73, 42, 0, 255, 127, 142}
-    ),
-    ((i8x16){0, 230, 255, 0, 255, 6, 106, 237, 230, 52, 223, 76, 0, 6, 127, 126})
-  );
   expect_vec(
     i8x16_avgr_u(
       (i8x16){0, 42, 255, 128, 127, 129, 6, 29, 103, 196, 231, 142, 17, 250, 1, 73},
@@ -1169,6 +1156,10 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   );
 
   // i16x8 arithmetic
+  expect_vec(
+    i16x8_abs((i16x8){0, 1, 42, -3, -56, 32767, -32768, 32766}),
+    ((i16x8){0, 1, 42, 3, 56, 32767, -32768, 32766})
+  );
   expect_vec(
     i16x8_neg((i16x8){0, 1, 42, -3, -56, 32767, -32768, 32766}),
     ((i16x8){0, -1, -42, 3, 56, -32767, -32768, -32766})
@@ -1263,7 +1254,8 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   );
 
   // i32x4 arithmetic
-  expect_vec(i32x4_neg((i32x4){0, 1, 0x80000000, 0x7fffffff}), ((i32x4){0, -1, 0x80000000, 0x80000001}));
+  expect_vec(i32x4_abs((i32x4){0, 1, 0x80000000, 0x80000001}), ((i32x4){0, 1, 0x80000000, 0x7fffffff}));
+  expect_vec(i32x4_neg((i32x4){0, 1, 0x80000000, 0x80000001}), ((i32x4){0, -1, 0x80000000, 0x7fffffff}));
   expect_eq(i32x4_any_true((i32x4){0, 0, 0, 0}), 0);
   expect_eq(i32x4_any_true((i32x4){0, 0, 1, 0}), 1);
   expect_eq(i32x4_any_true((i32x4){1, 0, 1, 1}), 1);
@@ -1339,20 +1331,8 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   // conversions
   expect_vec(i32x4_trunc_s_f32x4_sat((f32x4){42, NAN, INFINITY, -INFINITY}), ((i32x4){42, 0, 2147483647, -2147483648ll}));
   expect_vec(i32x4_trunc_u_f32x4_sat((f32x4){42, NAN, INFINITY, -INFINITY}), ((i32x4){42, 0, 4294967295ull, 0}));
-#ifdef __wasm_unimplemented_simd128__
-  expect_vec(i64x2_trunc_s_f64x2_sat((f64x2){42, NAN}), ((i64x2){42, 0}));
-  expect_vec(i64x2_trunc_s_f64x2_sat((f64x2){INFINITY, -INFINITY}), ((i64x2){9223372036854775807ll, -9223372036854775807ll - 1}));
-  expect_vec(i64x2_trunc_u_f64x2_sat((f64x2){42, NAN}), ((i64x2){42, 0}));
-  expect_vec(i64x2_trunc_u_f64x2_sat((f64x2){INFINITY, -INFINITY}), ((i64x2){18446744073709551615ull, 0}));
-#endif // __wasm_unimplemented_simd128__
   expect_vec(f32x4_convert_s_i32x4((i32x4){0, -1, 2147483647, -2147483647 - 1}), ((f32x4){0, -1, 2147483648., -2147483648.}));
   expect_vec(f32x4_convert_u_i32x4((i32x4){0, -1, 2147483647, -2147483647 - 1}), ((f32x4){0, 4294967296., 2147483648., 2147483648.}));
-#ifdef __wasm_unimplemented_simd128__
-  expect_vec(f64x2_convert_s_i64x2((i64x2){0, -1}), ((f64x2){0, -1}));
-  expect_vec(f64x2_convert_s_i64x2((i64x2){9223372036854775807, -9223372036854775807 - 1}), ((f64x2){9223372036854775807., -9223372036854775808.}));
-  expect_vec(f64x2_convert_u_i64x2((i64x2){0, -1}), ((f64x2){0, 18446744073709551616.}));
-  expect_vec(f64x2_convert_u_i64x2((i64x2){9223372036854775807 , -9223372036854775808.}), ((f64x2){9223372036854775807., 9223372036854775808.}));
-#endif // __wasm_unimplemented_simd128__
   expect_vec(
     i8x16_narrow_i16x8_s(
       (i16x8){129, 127, -32767, 32767, -32768, -1, 1, 0},
